Attribute VB_Name = "MathLuDecomp"
Option Explicit
Option Base 1

Public Function LuDet(A() As Double) As Double
    Dim N As Long, J As Long
    Dim Indx() As Long, D As Long
    Dim Det As Double
    
    N = UBound(A, 1)
    ReDim Indx(1 To N)
    
    Det = 1#
    LuDcmp A, Indx, D
    For J = 1 To N
        Det = Det * CDbl(D) * A(J, J)
    Next J
    
    LuDet = Det
    
End Function

Public Sub LuInv(A() As Double, B() As Double)
    Dim N As Long, I As Long, J As Long
    Dim D As Long
    Dim Col() As Double, Indx() As Long
    
    N = UBound(A, 1)
    ReDim Indx(1 To N), Col(1 To N)
    ReDim B(1 To N, 1 To N)
    
    LuDcmp A, Indx, D
    For J = 1 To N
        For I = 1 To N: Col(I) = 0#: Next I
        Col(J) = 1#
        LuBksb A, Indx, Col
        For I = 1 To N: B(I, J) = Col(I): Next I
    Next J
    
End Sub

Public Sub LuBksb(A() As Double, Indx() As Long, B() As Double)
    Dim N As Long
    Dim I As Long, II As Long, Ip As Long, J As Long
    Dim Sum As Double
    
    N = UBound(A, 1)
    
    For I = 1 To N
        Ip = Indx(I)
        Sum = B(Ip)
        B(Ip) = B(I)
        
        If II <> 0 Then
            For J = II To I - 1
                Sum = Sum - A(I, J) * B(J)
            Next J
        ElseIf Sum <> 0# Then
            II = I
        End If
        
        B(I) = Sum
    Next I
    
    For I = N To 1 Step -1
        Sum = B(I)
        
        For J = I + 1 To N
            Sum = Sum - A(I, J) * B(J)
        Next J
        
        B(I) = Sum / A(I, I)
    Next I
    
End Sub

Public Sub LuDcmp(A() As Double, Indx() As Long, D As Long)
    Const TINY As Double = 1E-20
    
    Dim N As Long
    Dim VV() As Double
    Dim I As Long, IMAX As Long, J As Long, K As Long
    Dim Big As Double, Dum As Double, Sum As Double, Temp As Double
    
    N = UBound(A, 1)
    ReDim VV(1 To N), Indx(1 To N)
    
    D = 1
  
    For I = 1 To N
        Big = 0#
        For J = 1 To N
            Temp = Abs(A(I, J))
            If Temp > Big Then Big = Temp
        Next J
        If Big = 0# Then Err.Raise ERR_NUM_LuDcmp, , ERR_DSC_LuDcmp
        VV(I) = 1# / Big
    Next I
  
    For J = 1 To N
    
        For I = 1 To J - 1
            Sum = A(I, J)
            For K = 1 To I - 1
                Sum = Sum - A(I, K) * A(K, J)
            Next K
            A(I, J) = Sum
        Next I
    
        Big = 0#
    
        For I = J To N
            Sum = A(I, J)
            For K = 1 To J - 1
                Sum = Sum - A(I, K) * A(K, J)
            Next K
            A(I, J) = Sum
            Dum = VV(I) * Abs(Sum)
            If Dum >= Big Then
                Big = Dum
                IMAX = I
            End If
        Next I
      
        If J <> IMAX Then
            For K = 1 To N
                Dum = A(IMAX, K): A(IMAX, K) = A(J, K): A(J, K) = Dum
            Next K
            D = -D
            VV(IMAX) = VV(J)
        End If
    
        Indx(J) = IMAX
    
        If A(J, J) = 0# Then A(J, J) = TINY
        
        If J <> N Then
            Dum = 1# / A(J, J)
            For I = J + 1 To N
                A(I, J) = A(I, J) * Dum
            Next I
        End If
      
    Next J

End Sub

Public Sub Mprove(A() As Double, Alud() As Double, Indx() As Long, _
                  B() As Double, x() As Double)
    Dim I As Long, J As Long, N As Long
    Dim Sdp As Double
    Dim R() As Double
    
    N = UBound(A, 1)
    ReDim R(1 To N)
    
    For I = 1 To N
        Sdp = -B(I)
        For J = 1 To N
            Sdp = Sdp + A(I, J) * x(J)
        Next J
        R(I) = Sdp
    Next I
    
    LuBksb Alud, Indx, R
    
    For I = 1 To N
        x(I) = x(I) - R(I)
    Next I
    
End Sub
                  

